@@ -4,10 +4,17 @@
 
 __author__ = """Stephen Wasilewski"""
 __email__ = 'stephanwaz@gmail.com'
-__version__ = '1.0.2'
+__version__ = '1.0.8'
 __all__ = ['plot', 'cl_plot']
 
 
+import matplotlib
+backend = matplotlib.get_backend()
+import clipt.plot as mplt
+matplotlib.use(backend)
+
+
+
 def get_root(inside=True, pyenv=False):
     """return root directory of clipt install"""
 
