@@ -32,8 +32,28 @@ def plot(ctx, config, outconfig, configalias, inputalias):
 
 main = plot # backwards compatibility
 
+
+coloropt = [
+          click.option('-fcol', default=0.0, type=float,
+                       help="colormap position for first color"),
+          click.option('-colors', default='med',
+                       callback=clk.color_inp,
+                       help="cmap name* or space seperated list of rgb tuples "
+                       "0,0,0 120,120,120 etc. if fewer than # series picks ."
+                       "from gradient includes ru_colors: 'blu', 'org', 'pur'"
+                       ", 'red', 'blg', 'mag', 'brn', 'grn', 'yel', 'ggr', "
+                       "'dark', 'med', 'light', 'xlight'"),
+          click.option('-positions', default=None,
+                       callback=clk.split_float,
+                       help="if cmap is a list positions are the mapping "
+                       "points else positions are the sample points used to "
+                       "remap"),
+          click.option('-step', default=None, type=int,
+                       help="steps for color map"),
+]
+
 # used by all
-shared = [
+shared = coloropt + [
           click.option('-drange', callback=clk.split_int,
                        help="index range for data series, if None gets all"),
           click.option('--ticklines/--no-ticklines', default=False,
@@ -47,23 +67,9 @@ shared = [
                        help="background color"),
           click.option('--legend/--no-legend', default=True,
                        help="include legend in plot"),
-          click.option('-colors', default='med',
-                       callback=clk.color_inp,
-                       help="cmap name* or space seperated list of rgb tuples "
-                       "0,0,0 120,120,120 etc. if fewer than # series picks ."
-                       "from gradient includes ru_colors: 'blu', 'org', 'pur'"
-                       ", 'red', 'blg', 'mag', 'brn', 'grn', 'yel', 'ggr', "
-                       "'dark', 'med', 'light', 'xlight'"),
           click.option('-comment', default='#',
                        help='regex "^[{}].*" indicating comment line '
                        'indicator'),
-          click.option('-positions', default=None,
-                       callback=clk.split_float,
-                       help="if cmap is a list positions are the mapping "
-                       "points else positions are the sample points used to "
-                       "remap"),
-          click.option('-step', default=None, type=int,
-                       help="steps for color map"),
           click.option('-labelweight', default='bold', type=click.Choice(['ultralight', 'light', 'normal', 'regular', 'book', 'medium', 'roman', 'semibold', 'demibold', 'demi', 'bold', 'heavy', 'extra bold', 'black']),
                        help="text weight for axes labels"),
           click.option('-width', default=10.5, type=float,
@@ -76,6 +82,8 @@ shared = [
                        help="background image for graph area"),
           click.option('-alpha', type=float, default=.5,
                        help="transparency for background"),
+          click.option('-format', default='.png',
+                       help="output file format for default naming (overridden by outf)"),
           click.option('--front/--no-front', default=False,
                        help="plot background in front of data"),
           click.option('--opts', '-opts', is_flag=True,
@@ -94,8 +102,6 @@ shared = [
 
 # used by bar, scatter, histo
 sharedA = [
-           click.option('-fcol', default=0.0, type=float,
-                        help="colormap position for first color"),
            click.option('--xgrid/--no-xgrid', default=False,
                         help="plot x grid lines"),
            click.option('--ygrid/--no-ygrid', default=False,
@@ -184,22 +190,7 @@ def bar(ctx, dataf, **kwargs):
             a1 = mgr.kwarg_match(mgr.read_data, kwargs)
             a1['autox'] = axext['xdata']
             xs, ys, labels = mgr.read_all_data(dataf, **a1)
-            if kwargs['xheader']:
-                a1['y_vals'] = [0]
-                a1['x_vals'] = []
-                a1['coerce'] = False
-                a1['xheader'] = False
-                a1['rows'] = False
-                _, xlabels, _ = mgr.read_data(dataf[0], **a1)
-                xlabels = xlabels[0]
-            else:
-                xlabels = []
-            xlabels = ruplot.get_labels(xlabels, kwargs['xlabels'])
-            labels = ruplot.get_labels(labels, kwargs['labels'])
-            if kwargs['rows']:
-                la = labels
-                labels = xlabels
-                xlabels = la
+            labels, xlabels = ruplot.get_labels(dataf, labels, a1, len(ys[0]), **kwargs)
             a3 = mgr.kwarg_match(ruplot.plot_setup, kwargs)
             ax, fig = ruplot.plot_setup(**a3)
             a4 = mgr.kwarg_match(ruplot.ticks, kwargs)
@@ -218,7 +209,7 @@ def bar(ctx, dataf, **kwargs):
             if kwargs['outf']:
                 outf = kwargs['outf']
             else:
-                outf = dataf[0].rsplit(".", 1)[0] + ".png"
+                outf = dataf[0].rsplit(".", 1)[0] + kwargs['format']
             a7 = mgr.kwarg_match(ruplot.plot_graph, kwargs)
             ruplot.plot_graph(fig, outf, handles=handles, **a7)
         except click.Abort:
@@ -288,7 +279,7 @@ def heatmap(ctx, dataf, **kwargs):
             if kwargs['outf']:
                 outf = kwargs['outf']
             else:
-                outf = dataf.rsplit(".", 1)[0] + ".png"
+                outf = dataf.rsplit(".", 1)[0] + kwargs['format']
             a7 = mgr.kwarg_match(ruplot.plot_graph, kwargs)
             ruplot.plot_graph(fig, outf, **a7)
         except click.Abort:
@@ -302,6 +293,8 @@ def heatmap(ctx, dataf, **kwargs):
 @click.argument('dataf', callback=clk.are_files)
 @click.option('-y_vals', default="-1", callback=clk.tup_int,
               help="index for yvals")
+@click.option('-weights', default=None, callback=clk.tup_int,
+              help="index for weights (if given, must be 1:1 match with y_vals)")
 @click.option('-axes', default="X,0,1,Y,0,ymax",
               help="enter as xname,xmin,xmax,yname,ymin,ymax - default uses"
               "min and max of data enter xmin etc to maintain autoscale")
@@ -345,22 +338,12 @@ def histo(ctx, dataf, **kwargs):
             a1 = mgr.kwarg_match(mgr.read_data, kwargs)
             a1['autox'] = axext['xdata']
             xs, ys, labels = mgr.read_all_data(dataf, **a1)
-            if kwargs['xheader']:
-                a1['y_vals'] = [0]
-                a1['x_vals'] = []
-                a1['coerce'] = False
-                a1['xheader'] = False
-                a1['rows'] = False
-                _, xlabels, _ = mgr.read_data(dataf[0], **a1)
-                xlabels = xlabels[0]
+            labels, xlabels = ruplot.get_labels(dataf, labels, a1, len(ys[0]), **kwargs)
+            if kwargs['weights'] is not None:
+                a1['y_vals'] = kwargs['weights']
+                _, weights, _ = mgr.read_all_data(dataf, **a1)
             else:
-                xlabels = []
-            xlabels = ruplot.get_labels(xlabels, kwargs['xlabels'])
-            labels = ruplot.get_labels(labels, kwargs['labels'])
-            if kwargs['rows']:
-                la = labels
-                labels = xlabels
-                xlabels = la
+                weights = None
             a3 = mgr.kwarg_match(ruplot.plot_setup, kwargs)
             ax, fig = ruplot.plot_setup(**a3)
             a4 = mgr.kwarg_match(ruplot.ticks, kwargs)
@@ -375,12 +358,13 @@ def histo(ctx, dataf, **kwargs):
                 a6['bins'] = kwargs['autobin']
             elif len(kwargs['bins']) == 1:
                 a6['bins'] = int(kwargs['bins'][0])
+            a6['weights'] = weights
             ax, handles = ruplot.plot_histo(ax, ys, labels, cmap,
                                             axext['ydata'], **a6)
             if kwargs['outf']:
                 outf = kwargs['outf']
             else:
-                outf = dataf[0].rsplit(".", 1)[0] + ".png"
+                outf = dataf[0].rsplit(".", 1)[0] + kwargs['format']
             a7 = mgr.kwarg_match(ruplot.plot_graph, kwargs)
             ruplot.plot_graph(fig, outf, handles=handles, **a7)
         except click.Abort:
@@ -403,8 +387,14 @@ def histo(ctx, dataf, **kwargs):
               callback=clk.tup_int, help="index for xvals")
 @click.option('-y_vals', default="-1", callback=clk.tup_int,
               help="index for yvals")
+@click.option('-area', default=None, callback=clk.tup_int, multiple=True,
+              help="give as single value (fills to y-min) or 2 values "
+              "(fills between) order given should match y_vals. colors"
+              " match line colors, but use the -falpha option")
 @click.option('-c_vals', default=None, callback=clk.tup_int,
-              help="optional index for color vals, only uses 1 index")
+              help="optional index for color vals")
+@click.option('-falpha', type=float, default=0.5,
+              help="alpha for -area(s)")
 @click.option('-m_vals', default=None, callback=clk.tup_int,
               help="optional index for marker size vals, only uses 1 index"
                    " values should be prescaled to marker size output")
@@ -442,6 +432,11 @@ def histo(ctx, dataf, **kwargs):
 @click.option('-criteria',
               help="plot markers based on criteria (either x=VAL or y=VAL)"
               "assumes single solution")
+@click.option('-y2', callback=clk.split_int,
+              help="idx positions (order from -y_vals) to plot on secondary y-axis")
+@click.option('-axes2', default="y,ymin,ymax",
+              help="enter as ymin,ymax - default uses"
+              "min and max of data from y2")
 @click.option('-lw', default="2", callback=clk.split_float,
               help="line weight if fewer than # series uses last")
 @clk.shared_decs(shared + sharedA + sharedB)
@@ -464,7 +459,7 @@ def scatter(ctx, dataf, **kwargs):
             xs, ys, labels = mgr.read_all_data(dataf, **a1)
             if kwargs['c_vals'] is not None:
                 a1['y_vals'] = kwargs['c_vals']
-                cs = mgr.read_all_data(dataf, **a1)[1]
+                _, cs, _ = mgr.read_all_data(dataf, **a1)
             else:
                 cs = None
             if kwargs['m_vals'] is not None:
@@ -472,12 +467,25 @@ def scatter(ctx, dataf, **kwargs):
                 msd = mgr.read_all_data(dataf, **a1)[1]
             else:
                 msd = None
-            labels = ruplot.get_labels(labels, kwargs['labels'])
+            areas = []
+            if kwargs['area'] is not None:
+                a1['x_vals'] = []
+                for a in kwargs['area']:
+                    a1['y_vals'] = a
+                    areas.append(mgr.read_all_data(dataf, **a1)[1])
+            labels = ruplot.get_user_labels(labels, kwargs['labels'])
             a3 = mgr.kwarg_match(ruplot.plot_setup, kwargs)
             ax, fig = ruplot.plot_setup(**a3)
             a4 = mgr.kwarg_match(ruplot.ticks, kwargs)
             a4.pop('labels', None)
-            ax = ruplot.tick_from_arg(ax, xs, ys, a4, kwargs)
+            if kwargs['y2'] is not None:
+                axes = kwargs['axes']
+                kwargs['axes'] = axes.rsplit(',', 3)[0] + ',' + kwargs['axes2']
+                ax2 = ax.twinx()
+                ax2 = ruplot.tick_from_arg(ax2, xs, [ys[i] for i in kwargs['y2']], a4, kwargs)
+                fig.sca(ax2)
+                kwargs['axes'] = axes
+            ax = ruplot.tick_from_arg(ax, xs, ys + areas, a4, kwargs)
             a5 = mgr.kwarg_match(ruplot.get_colors, kwargs)
             cmap = ruplot.get_colors(kwargs['colors'], **a5)
             a5['positions'] = kwargs['epositions']
@@ -486,18 +494,23 @@ def scatter(ctx, dataf, **kwargs):
             a6 = mgr.kwarg_match(ruplot.plot_scatter, kwargs)
             a6.pop('labels', None)
             a6['legend'] = kwargs['legend'] and not kwargs['areaonly']
-            ax, handles = ruplot.plot_scatter(fig, ax, xs, ys, labels, cmap,
-                                              emap=emap, cs=cs, msd=msd, **a6)
+            ax, handles, handles2 = ruplot.plot_scatter(fig, ax, xs, ys, labels, cmap,
+                                                        emap=emap, cs=cs, msd=msd,
+                                                        areas=areas, **a6)
             if kwargs['diagonal']:
                 ax.plot(ax.get_xlim(), ax.get_ylim(), lw=0.6, ls='--', color='black', ms=0)
             if kwargs['outf']:
                 outf = kwargs['outf']
             else:
-                outf = dataf[0].rsplit(".", 1)[0] + ".png"
+                outf = dataf[0].rsplit(".", 1)[0] + kwargs['format']
             a7 = mgr.kwarg_match(ruplot.plot_graph, kwargs)
             if cs is not None:
                 a7['legend'] = False
-            ruplot.plot_graph(fig, outf, handles=handles, **a7)
+            if len(handles2) > 0:
+                bba = (1.1, 1)
+            else:
+                bba = (1.05, 1)
+            ruplot.plot_graph(fig, outf, handles=handles, handles2=handles2, bbox_to_anchor=bba, **a7)
         except click.Abort:
             raise
         except Exception as ex:
@@ -536,8 +549,6 @@ def scatter(ctx, dataf, **kwargs):
 @click.option('--xheader/--no-xheader', default=False,
               help="indicates that data has a header column to get x-axis "
               "labels (overridden by xlabels)")
-@click.option('-fcol', default=0.0, type=float,
-              help="colormap position for first color")
 @click.option('--xgrid/--no-xgrid', default=False,
               help="plot x grid lines")
 @click.option('--ygrid/--no-ygrid', default=False,
@@ -574,28 +585,13 @@ def box(dataf, **kwargs):
             a1 = mgr.kwarg_match(mgr.read_data, kwargs)
             a1['autox'] = [0, 1]
             xs, ys, labels = mgr.read_all_data(dataf, **a1)
-            if kwargs['xheader']:
-                a1['y_vals'] = [0]
-                a1['x_vals'] = []
-                a1['coerce'] = False
-                a1['xheader'] = False
-                a1['rows'] = False
-                _, xlabels, _ = mgr.read_data(dataf[0], **a1)
-                xlabels = xlabels[0]
-            else:
-                xlabels = []
-            xlabels = ruplot.get_labels(xlabels, kwargs['xlabels'])
-            labels = ruplot.get_labels(labels, kwargs['labels'])
-            if kwargs['rows']:
-                la = labels
-                labels = xlabels
-                xlabels = la
+            labels, xlabels = ruplot.get_labels(dataf, labels, a1, len(ys[0]), **kwargs)
             a3 = mgr.kwarg_match(ruplot.plot_setup, kwargs)
             ax, fig = ruplot.plot_setup(**a3)
             a4 = mgr.kwarg_match(ruplot.ticks, kwargs)
             a4.pop('labels', None)
             a4['xlabels'] = xlabels
-            ax = ruplot.tick_from_arg(ax, [0, len(ys)], ys, a4, kwargs)
+            ax = ruplot.tick_from_arg(ax, [0, len(ys[0])], ys, a4, kwargs)
             a5 = mgr.kwarg_match(ruplot.get_colors, kwargs)
             cmap = ruplot.get_colors(kwargs['colors'], **a5)
             a6 = mgr.kwarg_match(ruplot.plot_box, kwargs)
@@ -605,7 +601,7 @@ def box(dataf, **kwargs):
             if kwargs['outf']:
                 outf = kwargs['outf']
             else:
-                outf = dataf[0].rsplit(".", 1)[0] + ".png"
+                outf = dataf[0].rsplit(".", 1)[0] + kwargs['format']
             a7 = mgr.kwarg_match(ruplot.plot_graph, kwargs)
             ruplot.plot_graph(fig, outf, handles=handles, **a7)
         except click.Abort:
@@ -619,7 +615,9 @@ def box(dataf, **kwargs):
 @click.argument('dataf', callback=clk.are_files)
 @click.option('-y_vals', default="-1", callback=clk.tup_int,
               help="index for yvals")
-@click.option('-axes', default="X,0,1,Y,0,ymax",
+@click.option('-weights', default=None, callback=clk.tup_int,
+              help="index for weights (if given, must be 1:1 match with y_vals)")
+@click.option('-axes', default="X,0,1,Y,ymin,ymax",
               help="enter as xname,xmin,xmax,yname,ymin,ymax - default uses"
               "min and max of data enter xmin etc to maintain autoscale")
 @click.option('-rwidth', default=0.95, type=float,
@@ -632,16 +630,20 @@ def box(dataf, **kwargs):
 @click.option('-clw', default=1.,
               help="linewidth for median line")
 @click.option('-series', default=1,
-              help="number of series to plot data in")
+              help="number of series to pl ot data in")
 @click.option('--clbg/--no-clbg', default=True,
               help="median line uses -bg")
-@click.option('-fillalpha', default=1.0,
+@click.option('-conf', default=None, type=float,
+              help="plot confidence interval on data")
+@click.option('-confm', default=None, type=float,
+              help="plot confidence interval on mean of data")
+@click.option('-kernelwidth', '-kw', default=.5,
+              help="scale factor to apply to kernel bandwidth selector (1 uses scotts rule)")
+@click.option('-fillalpha', default=.5,
               help="alpha for fill color (matches line color)")
 @click.option('--xheader/--no-xheader', default=False,
               help="indicates that data has a header column to get x-axis "
               "labels (overridden by xlabels)")
-@click.option('-fcol', default=0.0, type=float,
-              help="colormap position for first color")
 @click.option('--xgrid/--no-xgrid', default=False,
               help="plot x grid lines")
 @click.option('--ygrid/--no-ygrid', default=False,
@@ -678,38 +680,32 @@ def violin(dataf, **kwargs):
             a1 = mgr.kwarg_match(mgr.read_data, kwargs)
             a1['autox'] = [0, 1]
             xs, ys, labels = mgr.read_all_data(dataf, **a1)
-            if kwargs['xheader']:
-                a1['y_vals'] = [0]
-                a1['x_vals'] = []
-                a1['coerce'] = False
-                a1['xheader'] = False
-                a1['rows'] = False
-                _, xlabels, _ = mgr.read_data(dataf[0], **a1)
-                xlabels = xlabels[0]
+            labels, xlabels = ruplot.get_labels(dataf, labels, a1, len(ys[0]), **kwargs)
+            if kwargs['weights'] is not None:
+                a1['y_vals'] = kwargs['weights']
+                _, weights, _ = mgr.read_all_data(dataf, **a1)
             else:
-                xlabels = []
-            xlabels = ruplot.get_labels(xlabels, kwargs['xlabels'])
-            labels = ruplot.get_labels(labels, kwargs['labels'])
-            if kwargs['rows']:
-                la = labels
-                labels = xlabels
-                xlabels = la
+                weights = None
             a3 = mgr.kwarg_match(ruplot.plot_setup, kwargs)
             ax, fig = ruplot.plot_setup(**a3)
             a4 = mgr.kwarg_match(ruplot.ticks, kwargs)
             a4.pop('labels', None)
             a4['xlabels'] = xlabels
-            ax = ruplot.tick_from_arg(ax, [0, len(ys)], ys, a4, kwargs)
+            ax = ruplot.tick_from_arg(ax, [0, len(ys[0])], ys, a4, kwargs)
             a5 = mgr.kwarg_match(ruplot.get_colors, kwargs)
             cmap = ruplot.get_colors(kwargs['colors'], **a5)
-            a6 = mgr.kwarg_match(ruplot.plot_box, kwargs)
+            a6 = mgr.kwarg_match(ruplot.plot_violin, kwargs)
             a6.pop('labels', None)
+            a6['weights'] = weights
             ax, handles = ruplot.plot_violin(ax, ys, labels, cmap,
                                              axext['ydata'], **a6)
+            a4['xdata'] = ax.get_xlim()
+            a4['ydata'] = ax.get_ylim()
+            ax = ruplot.ticks(ax, **a4)
             if kwargs['outf']:
                 outf = kwargs['outf']
             else:
-                outf = dataf[0].rsplit(".", 1)[0] + ".png"
+                outf = dataf[0].rsplit(".", 1)[0] + kwargs['format']
             a7 = mgr.kwarg_match(ruplot.plot_graph, kwargs)
             ruplot.plot_graph(fig, outf, handles=handles, **a7)
         except click.Abort:
@@ -719,6 +715,54 @@ def violin(dataf, **kwargs):
     return 'violin', kwargs
 
 
+@plot.command('previewpal')
+@click.option('-n', default=4,
+              help="number of series to draw from colormap")
+@click.option('--colorbar/--series', default=False,
+              help="whether to preview colorbar (continuous) or series"
+              " (category) scale)")
+@click.option('-outf', default='previewpal.png')
+@click.option('--opts', '-opts', is_flag=True,
+             help="check parsed options")
+@click.option('--debug', is_flag=True,
+             help="show traceback on exceptions")
+@clk.shared_decs(coloropt)
+def previewpal(**kwargs):
+    """
+    create plot color palettes and formatted rgbs
+    """
+    if kwargs['opts']:
+        kwargs['opts'] = False
+        clk.echo_args(**kwargs)
+    else:
+        try:
+            step = kwargs['step']
+            positions = kwargs['positions']
+            fcol = kwargs['fcol']
+            n = kwargs['n']
+            colors = kwargs['colors']
+            cmap = ruplot.get_colors(colors, step=step, positions=positions)
+            ax, fig = ruplot.plot_setup(areaonly=True)
+            if kwargs['colorbar']:
+                ruplot.add_colorbar(fig, cmap, axes = [0, 0, 1, 1])
+                ax.remove()
+            else:
+                inc = ruplot.color_inc(fcol, step, n)
+                for i in range(n):
+                    cinc = ruplot.color_inc_i(fcol, i, n, step)
+                    c = cmap.to_rgba(cinc)
+                    try:
+                        p = .05 + i * .9/(n-1)
+                    except ZeroDivisionError:
+                        p = .5
+                    ax.plot([p,p], [0,1], color=c, lw=4)
+            ruplot.plot_graph(fig, kwargs['outf'], width=4, height=2,
+                              areaonly=True)
+        except click.Abort:
+            raise
+        except Exception as ex:
+            clk.print_except(ex, kwargs['debug'])
+    return 'previewpal', kwargs
 
 
 @plot.command('colors')
@@ -738,6 +782,7 @@ def violin(dataf, **kwargs):
               help="check parsed options")
 @click.option('--debug', is_flag=True,
               help="show traceback on exceptions")
+@clk.shared_decs(coloropt)
 def colors(**kwargs):
     """
     create plot color palettes and formatted rgbs
@@ -792,7 +837,7 @@ def colors(**kwargs):
             raise
         except Exception as ex:
             clk.print_except(ex, kwargs['debug'])
-    return 'box', kwargs
+    return 'colors', kwargs
 
 
 @plot.resultcallback()
