@@ -27,6 +27,8 @@ from matplotlib.patches import Circle, Patch
 import matplotlib.colors as colors
 import matplotlib.cm as cmx
 from matplotlib.transforms import Bbox
+import clasp.script_tools as mgr
+from hdrstats import hdrstats as hs
 
 
 daycount = [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334]
@@ -85,13 +87,13 @@ def tick_from_arg(ax, xs, ys, a4, kwargs):
     return ax
 
 
-def get_labels(labels, mlab):
+def get_user_labels(labels, mlab):
     """returns data based on standard argument parsing
 
     Parameters
     ----------
     labels: list
-        labels retruned by read_all_data
+        labels returned by read_all_data
     mlab: list
         manual labels for data
     Returns
@@ -108,8 +110,39 @@ def get_labels(labels, mlab):
     return labels
 
 
-def get_axes(arg, xs, ys, polar=False, polarauto=True, stacked=False, pery=False):
-    """parse axes string argument xname,xmin,xmax,yname,ymin,ymax"""
+def get_labels(dataf, labs, a1, ycnt, xheader=False, xlabels=None,
+               labels=None, rows=False, drange=None, y_vals=[-1,],
+               **kwargs):
+    if xheader:
+        a1['y_vals'] = [0]
+        a1['x_vals'] = []
+        a1['coerce'] = False
+        a1['xheader'] = False
+        a1['drange'] = None
+        a1['rows'] = False
+        _, xlabs, _ = mgr.read_data(dataf[0], **a1)
+        xlabs = xlabs[0]
+    else:
+        xlabs = []
+    xlabs = get_user_labels(xlabs, xlabels)
+    labs = get_user_labels(labs, labels)
+    if rows:
+        if drange is not None:
+            la = [labs[i] for i in drange]
+        else:
+            la = labs[:ycnt]
+        labs = []
+        for i in y_vals:
+            try:
+                labs.append(xlabs[i[1]])
+            except TypeError:
+                labs.append(xlabs[i])
+        xlabs = la
+    elif drange is not None:
+        xlabs = [xlabs[i] for i in drange]
+    return labs, xlabs
+
+def ax_limits(x):
     def smin(x):
         try:
             return min(x)
@@ -121,6 +154,11 @@ def get_axes(arg, xs, ys, polar=False, polarauto=True, stacked=False, pery=False
             return max(x)
         except Exception:
             return None
+    return smin(x), smax(x)
+
+
+def get_axes(arg, xs, ys, polar=False, polarauto=True, stacked=False, pery=False, **kwargs):
+    """parse axes string argument xname,xmin,xmax,yname,ymin,ymax"""
     x = flat(xs)
     if stacked:
         try:
@@ -129,7 +167,7 @@ def get_axes(arg, xs, ys, polar=False, polarauto=True, stacked=False, pery=False
             y = []
     else:
         y = flat(ys)
-    axes = ['X', smin(x), smax(x), 'Y', smin(y), smax(y)]
+    axes = ['X', *ax_limits(x), 'Y', *ax_limits(y)]
     try:
         ax = arg.split(",")
         for i, v in enumerate(ax):
@@ -154,7 +192,7 @@ def get_axes(arg, xs, ys, polar=False, polarauto=True, stacked=False, pery=False
     return naxes
 
 
-def plot_setup(fg='black', bg='white', polar=False, areaonly=False, params={}):
+def plot_setup(fg='black', bg='white', polar=False, areaonly=False, params={}, **kwargs):
     """
     setup plot with uniform styling and create axes for plot
 
@@ -191,11 +229,11 @@ def plot_setup(fg='black', bg='white', polar=False, areaonly=False, params={}):
 
 
 def ticks(ax, xdata=[0, 1], ydata=[0, 1], tcol='black', labels=['X', 'Y'],
-          xgrid=False, ygrid=False, xscale='linear', yscale='linear',
+          xgrid=True, ygrid=True, xscale='linear', yscale='linear',
           annualx=False, dayy=False, pery=False, ticklines=False, pph=1,
           bottom=None, bg='white', xlabels=None, dpy=365, hpd=24, sh=0,
-          polar=False, xticks=None, yticks=None, labelweight='bold',
-          matchxy=False):
+          polar=False, xticks=None, yticks=None, labelweight='ultralight',
+          matchxy=False, **kwargs):
     """
     setup ticks/axes for plot
 
@@ -236,8 +274,11 @@ def ticks(ax, xdata=[0, 1], ydata=[0, 1], tcol='black', labels=['X', 'Y'],
         xmax = max(xmax, ymax)
         ymin = xmin
         ymax = xmax
+    tcol = colors.to_rgba(tcol)
     ax.xaxis.grid(linestyle="-", linewidth=0.3, color=tcol)
     ax.yaxis.grid(linestyle="-", linewidth=0.3, color=tcol, zorder=1)
+    ax.xaxis.grid(linestyle="--", linewidth=0.3, color=tcol, alpha=.2, which='minor')
+    ax.yaxis.grid(linestyle="--", linewidth=0.3, color=tcol, alpha=.2, zorder=1, which='minor')
     ax.xaxis.grid(xgrid)
     ax.yaxis.grid(ygrid)
     ax.set_xlim(left=xmin, right=xmax)
@@ -300,17 +341,35 @@ def ticks(ax, xdata=[0, 1], ydata=[0, 1], tcol='black', labels=['X', 'Y'],
     if bottom is None:
         bottom = 0
     if yticks is not None:
-        ax.set_yticks(np.append(np.arange(bottom+ymin, ymax+bottom,
-                      old_div((ymax-ymin),(yticks))),ymax+bottom))
+        if yscale != 'log':
+            ax.set_yticks(np.append(np.arange(bottom+ymin, ymax+bottom,
+                          (ymax-ymin)/yticks),ymax+bottom))
+        else:
+            decs = np.ceil(np.log10(ymax) - np.log10(ymin))
+            maj = np.power(10, np.log10(ymin) + np.arange(decs+1))
+            minors = np.meshgrid(1 + np.arange(yticks-1)*10/yticks, maj[:-1])
+            minors = (minors[0] * minors[1]).flatten()
+            ax.set_yticks(maj)
+            ax.set_yticks(minors, minor=True)
+            ax.yaxis.grid(ygrid, which='minor')
     if xticks is not None:
-        ax.set_xticks(np.append(np.arange(xmin, xmax, old_div((xmax-xmin),(xticks))),xmax))
+        if xscale != 'log':
+            ax.set_xticks(np.append(np.arange(xmin, xmax, (xmax-xmin)/xticks),xmax))
+        else:
+            decs = np.ceil(np.log10(xmax) - np.log10(xmin))
+            maj = np.power(10, np.log10(xmin) + np.arange(decs+1))
+            minors = np.meshgrid(1 + np.arange(xticks-1)*10/xticks, maj[:-1])
+            minors = (minors[0] * minors[1]).flatten()
+            ax.set_xticks(maj)
+            ax.set_xticks(minors, minor=True)
+            ax.xaxis.grid(xgrid, which='minor')
     return ax
 
 
-def plot_legend(handles, bbox_to_anchor=(1.05, 1), loc=2,
-                fg='black', bg='white'):
+def plot_legend(ax, handles, bbox_to_anchor=(1.05, 1), loc=2,
+                fg='black', bg='white', title=None):
     """add legend to figure"""
-    leg = plt.legend(handles=handles, frameon=False,
+    leg = ax.legend(handles=handles, frameon=False, title=title,
                      bbox_to_anchor=bbox_to_anchor, loc=loc, facecolor=bg)
     for text in leg.get_texts():
         text.set_color(fg)
@@ -319,7 +378,7 @@ def plot_legend(handles, bbox_to_anchor=(1.05, 1), loc=2,
 def add_colorbar(fig, pc, axes=[0.3, 0.0, 0.4, 0.02],
                  ticks=None, ticklabels=None, orientation='horizontal'):
     """add colorbar scale to figure below plot"""
-    cbaxes = fig.add_axes(axes)
+    cbaxes = fig.add_axes(axes, label='colorbar')
     if ticklabels is not None:
         if ticks is None:
             vmin, vmax = pc.get_clim()
@@ -336,14 +395,21 @@ def add_colorbar(fig, pc, axes=[0.3, 0.0, 0.4, 0.02],
         t.set_visible(False)
 
 
-def plot_graph(fig, saveimage, width=10.5, height=5, bg='white', fg='black',
-               handles=[], dpi=200, bbox_to_anchor=(1.05, 1),
+def plot_graph(fig, saveimage, width=5, height=5, bg='white', fg='black',
+               handles=[], handles2=[], dpi=200, bbox_to_anchor=(1.05, 1),
                loc=2, legend=False, background=None,
-               front=False, alpha=.5, areaonly=False, polar=False):
+               front=False, alpha=.5, areaonly=False, polar=False, **kwargs):
     """add legend and save image of plot"""
     fig.set_size_inches(width, height)
     if legend and not areaonly:
-        plot_legend(handles=handles, bbox_to_anchor=bbox_to_anchor,
+        try:
+            if isinstance(fig.axes[1], matplotlib.axes.SubplotBase):
+                plot_legend(fig.axes[1], handles=handles2,
+                            bbox_to_anchor=(bbox_to_anchor[0], 0),
+                            loc=3, bg=bg, fg=fg, title='right axis')
+        except IndexError:
+            pass
+        plot_legend(fig.axes[0], handles=handles, bbox_to_anchor=bbox_to_anchor,
                     loc=loc, bg=bg, fg=fg)
     ax = fig.axes[0]
     if areaonly:
@@ -369,7 +435,7 @@ def plot_graph(fig, saveimage, width=10.5, height=5, bg='white', fg='black',
     plt.close()
 
 
-def get_colors(cmap, step=None, positions=None, funcs=[]):
+def get_colors(cmap, step=None, positions=None, funcs=[], **kwargs):
     """get colormap from cmap name, color list or CliptColors spec
 
     Parameters
@@ -503,27 +569,64 @@ def plot_criteria(ax, x, y, criteria, flipxy=False,
         ax.plot(filtx, filty, **kwargs)
 
 
+def color_inc_i(fcol, i, n, step):
+    if step is not None:
+        try:
+            fcol = int(step*fcol % step)/step
+        except ZeroDivisionError:
+            fcol = 0
+        cinc = (fcol + (i+.5)/step) % 1
+    else:
+        try:
+            inc = old_div((1.-fcol),(n-1))
+            cinc = fcol + i*inc
+        except ZeroDivisionError:
+            cinc = fcol + i
+    return cinc
+
+
 def plot_scatter(fig, ax, xs, ys, labels, colormap, criteria=None, lw=2, ms=0,
-                 mrk='o', step=None, fcol=0.0, mew=0.0, emap=None,
-                 flipxy=False, cs=None, cmin=None, cmax=None,
-                 msd=None, mmin=None, mmax=None, legend=True, polar=False,
-                 **kwargs):
+                 mrk='o', step=None, fcol=0.0, mew=0.0, emap=None, estep=None,
+                 flipxy=False, cs=None, cmin=None, cmax=None, y2=None,
+                 msd=None, mmin=None, mmax=None, legend=True,
+                 polar=False, areas=[], falpha=0.5, **kwargs):
     """adds scatterplots/lines to ax and returns ax and handles for legend"""
     nlab = len(labels)
     for i in range(len(ys)):
         if i >= nlab:
             labels.append("series{:02d}".format(i))
     handles = []
-    if step is not None:
-        inc = old_div((1.-fcol),(step-1))
-    else:
-        try:
-            inc = old_div((1.-fcol),(len(ys)-1))
-        except ZeroDivisionError:
-            inc = 1
     if emap is None:
         emap = colormap
-    for i, (x, y, l) in enumerate(zip(xs, ys, labels)):
+    if y2 is None:
+        y2 = []
+    else:
+        ax2 = fig.axes[1]
+    if cs is None:
+        cvals = (None,) * len(xs)
+    else:
+        if cmax is None:
+            cmax = max(flat(cs))
+        if cmin is None:
+            cmin = min(flat(cs))
+        j = 0
+        cvals = []
+        for i in range(len(ys)):
+            if get_nth(ms, i) > 0:
+                cvals.append(get_nth(cs, j))
+                j += 1
+            else:
+                cvals.append(None)
+    for i, (x, y, l, cs) in enumerate(zip(xs, ys, labels, cvals)):
+        if i in y2:
+            axT = ax2
+            ax2.set_zorder(ax.get_zorder()+1)
+        else:
+            axT = ax
+            try:
+                ax.set_zorder(ax2.get_zorder()+1)
+            except UnboundLocalError:
+                pass
         lwa = get_nth(lw, i)
         if msd is not None:
             if mmax is None:
@@ -535,23 +638,24 @@ def plot_scatter(fig, ax, xs, ys, labels, colormap, criteria=None, lw=2, ms=0,
             msa = get_nth(ms, i)
         mka = get_nth_loop(mrk, i)
         mewa = get_nth_loop(mew, i)
-        if step is not None or inc > 1:
-            cinc = (fcol + i*inc) % 1
-        else:
-            cinc = fcol + i*inc
+        cinc = color_inc_i(fcol, i, len(ys), step)
         c = colormap.to_rgba(cinc)
-        mec = emap.to_rgba(cinc)
+        ecinc = color_inc_i(fcol, i, len(ys), estep)
+        mec = emap.to_rgba(ecinc)
+        if i < len(areas):
+            if len(areas[i]) < 2:
+                ya2 = axT.axes.get_ylim()[0]
+            else:
+                ya2 = areas[i][1]
+            axT.fill_between(x, areas[i][0], ya2, alpha=falpha, color=c, zorder=-1,
+                            linestyle='--', linewidth=lwa/2)
         if cs is not None:
-            if cmax is None:
-                cmax = max(flat(cs))
-            if cmin is None:
-                cmin = min(flat(cs))
             plotargs = {'linewidth': lwa, 's': msa**2, 'label': l,
                         'marker': mka, 'cmap': colormap.cmap, 'linewidth': mewa,
-                        'vmin': cmin, 'vmax': cmax, 'c': cs[i], 'edgecolors': mec,
+                        'vmin': cmin, 'vmax': cmax, 'c': cs, 'edgecolors': mec,
                         'norm': colormap.norm}
             plotargs.update(kwargs)
-            ax.scatter(x, y, **plotargs)
+            axT.scatter(x, y, **plotargs)
             if legend:
                 pc = cmx.ScalarMappable(norm=colormap.norm, cmap=colormap.cmap)
                 if polar:
@@ -563,22 +667,26 @@ def plot_scatter(fig, ax, xs, ys, labels, colormap, criteria=None, lw=2, ms=0,
                         'marker': mka, 'linewidth': mewa, 'c': [c],
                         'edgecolors': mec}
             plotargs.update(kwargs)
-            ax.scatter(x, y, **plotargs)
+            axT.scatter(x, y, **plotargs)
         else:
             plotargs = {'linewidth': lwa, 'markersize': msa, 'label': l,
                         'marker': mka, 'color': c, 'mfc': c, 'mec': mec,
                         'mew': mewa}
             plotargs.update(kwargs)
             if flipxy:
-                ax.plot(y, x, **plotargs)
+                axT.plot(y, x, **plotargs)
             else:
-                ax.plot(x, y, **plotargs)
+                axT.plot(x, y, **plotargs)
         if criteria:
             copts = {'linewidth': 0, 'mfc': c, 'mew': 0,
                      'markersize': lwa*2.5, 'marker': 'o'}
             plot_criteria(ax, x, y, criteria, flipxy, copts)
-    handles, labels = ax.get_legend_handles_labels()
-    return ax, handles
+    handles, _ = ax.get_legend_handles_labels()
+    try:
+        handles2, _ = ax2.get_legend_handles_labels()
+    except UnboundLocalError:
+        handles2 = []
+    return ax, handles, handles2
 
 
 def plot_heatmap(fig, ax, data, colormap, vmin=None, vmax=None,
@@ -607,8 +715,8 @@ def plot_heatmap(fig, ax, data, colormap, vmin=None, vmax=None,
     return ax
 
 
-def plot_bar(ax, xs, ys, labels, colormap, stacked=False, rwidth=.8, step=None,
-             bwidth=.9, brng=[0, 1], bottom=0, polar=False, polar0=False,
+def plot_bar(ax, xs, ys, labels, colormap, stacked=False, rwidth=.8, step=None, estep=None,
+             bwidth=.9, fcol=0.0, brng=[0, 1], bottom=0, polar=False, polar0=False,
              emap=None, ew=[0], **kwargs):
     """adds bar plots to ax and returns ax and handles for legend"""
     nlab = len(labels)
@@ -621,13 +729,6 @@ def plot_bar(ax, xs, ys, labels, colormap, stacked=False, rwidth=.8, step=None,
         nlab = 1
     else:
         nlab = len(labels)
-    if step is not None:
-        inc = 1./(step-1)
-    else:
-        try:
-            inc = 1./(len(ys)-1)
-        except ZeroDivisionError:
-            inc = 1
     xl = ax.axes.get_xlim()
     xlim = [xl[0]+brng[0]*(xl[1]-xl[0]), xl[0]+brng[1]*(xl[1]-xl[0])]
     xrng = abs(xlim[1]-xlim[0])/float(len(xs[0]))
@@ -644,9 +745,12 @@ def plot_bar(ax, xs, ys, labels, colormap, stacked=False, rwidth=.8, step=None,
         off0 = old_div(xrng,2)-old_div(width*(nlab-1),2) + xlim[0]
     w2 = width*bwidth
     bot = 0
+    n = len(ys)
     for i, (x, y, l) in enumerate(zip(xs, ys, labels)):
-        c = colormap.to_rgba(i*inc)
-        e = emap.to_rgba(i*inc)
+        cinc = color_inc_i(fcol, i, n, step)
+        c = colormap.to_rgba(cinc)
+        einc = color_inc_i(fcol, i, n, estep)
+        e = emap.to_rgba(einc)
         ewa = get_nth_loop(ew, i)
         plotargs = {'label': l, 'color': c, 'linewidth': ewa, 'edgecolor': e}
         plotargs.update(kwargs)
@@ -663,7 +767,7 @@ def plot_bar(ax, xs, ys, labels, colormap, stacked=False, rwidth=.8, step=None,
 
 
 def plot_box(ax, data, labels, colormap, ylim, rwidth=.8, step=None, mark='x',
-             mew=0.5, ms=3.0, lw=1.0, clw=1.0, clbg=True, fillalpha=1.0, notch=False,
+             mew=0.5, ms=3.0, lw=1.0, fcol=0.0, clw=1.0, clbg=True, fillalpha=1.0, notch=False,
              series=1, bg='white', inline=False, mean=False, **kwargs):
     """adds box plots to ax and returns ax and handles for legend"""
     nlab = len(labels)
@@ -671,18 +775,11 @@ def plot_box(ax, data, labels, colormap, ylim, rwidth=.8, step=None, mark='x',
         if i >= nlab:
             labels.append("series{:02d}".format(i))
     nlab = len(labels)
-    if step is not None:
-        inc = 1./(step-1)
-    else:
-        try:
-            inc = 1./(series-1)
-        except ZeroDivisionError:
-            inc = 1
     chunksize = int(len(data)/series)
     handles = []
-    
     for i in range(series):
-        c = colormap.to_rgba(i*inc)
+        cinc = color_inc_i(fcol, i, nlab, step)
+        c = colormap.to_rgba(cinc)
         if clbg:
             medianc = bg
         else:
@@ -715,26 +812,20 @@ def plot_box(ax, data, labels, colormap, ylim, rwidth=.8, step=None, mark='x',
     return ax, handles
 
 
-def plot_violin(ax, data, labels, colormap, ylim, rwidth=.8, step=None, lw=1.0,
-                clw=1.0, clbg=True, fillalpha=1.0, median=True,
-                series=1, bg='white', inline=False, mean=False, **kwargs):
+def plot_violin(ax, data, labels, colormap, ylim, rwidth=.8, step=None, lw=1.0, kernelwidth=.5,
+                clw=1.0, clbg=True, fcol=0.0, fillalpha=1.0, median=True, conf=None, confm=None,
+                series=1, bg='white', inline=False, mean=False, weights=None, **kwargs):
     """adds violin plots to ax and returns ax and handles for legend"""
     nlab = len(labels)
     for i in range(series):
         if i >= nlab:
             labels.append("series{:02d}".format(i))
     nlab = len(labels)
-    if step is not None:
-        inc = 1./(step-1)
-    else:
-        try:
-            inc = 1./(series-1)
-        except ZeroDivisionError:
-            inc = 1
     chunksize = int(len(data)/series)
     handles = []
     for i in range(series):
-        c = colormap.to_rgba(i*inc)
+        cinc = color_inc_i(fcol, i, nlab, step)
+        c = colormap.to_rgba(cinc)
         if clbg:
             medianc = bg
         else:
@@ -745,44 +836,71 @@ def plot_violin(ax, data, labels, colormap, ylim, rwidth=.8, step=None, lw=1.0,
         else:
             x = np.arange(i, len(data), series)
             sw = 1
-        
-        vplot = ax.violinplot(data[i*chunksize:i*chunksize+chunksize],
-                              showmeans=mean, showmedians=median,
-                              widths=rwidth*sw, positions=x)
+        ds = np.array(data[i*chunksize:i*chunksize+chunksize])
+        if weights is not None:
+            ws = np.array(weights[i*chunksize:i*chunksize+chunksize])
+        else:
+            ws = (None,) * len(ds)
+        vstats = []
+        for d, w in zip(ds, ws):
+            vstats.append(hs.kernel(d, w=w, n=1000, bws=kernelwidth))
+        vplot = ax.violin(vstats, showmeans=mean, showmedians=median,
+                          widths=rwidth*sw, positions=x)
+        if confm is not None:
+            bstats = hs.conf_box(ds, ws, confm)
+            plotargs = {
+                'boxprops' : {'linewidth':clw, 'color':c, 'linestyle':'--', 'dash_joinstyle':'miter'},
+                'medianprops' : {'linewidth':0,},
+                'whiskerprops': {'linewidth':0,}
+            }
+            blot = ax.bxp(bstats, widths=rwidth*sw/2, positions=x, showfliers=False, showcaps=False, **plotargs)
+        if conf is not None:
+            bstats = hs.quant_box(ds, ws, conf)
+            plotargs = {
+                'boxprops' : {'linewidth':lw, 'color':c},
+                'medianprops' : {'linewidth':0,},
+                'whiskerprops': {'linewidth':0,}
+            }
+            blot = ax.bxp(bstats, widths=rwidth*sw/4, positions=x, showfliers=False, showcaps=False, **plotargs)
         for vp in vplot['bodies']:
             vp.set_facecolor(c)
             vp.set_alpha(fillalpha)
-        for vp in [ vplot[i] for i in ['cmins', 'cmaxes', 'cbars', 'cmeans']]:
-            vp.set_edgecolor(c)
-            vp.set_linewidth(lw)
-        for vp in [ vplot[i] for i in ['cmedians']]:
-            vp.set_edgecolor(c)
-            vp.set_linewidth(clw)
-        vplot['cmeans'].set_linestyle(':')
+        for j in ['cmins', 'cmaxes', 'cbars', 'cmeans', 'cmedians']:
+            try:
+                vp = vplot[j]
+            except KeyError:
+                pass
+            else:
+                if j == 'cmedians':
+                    vp.set_edgecolor(medianc)
+                    vp.set_linewidth(clw)
+                else:
+                    vp.set_edgecolor(c)
+                    vp.set_linewidth(lw)
+                
+                if j == 'cmeans':
+                    vp.set_linestyle(':')
+                    vp.set_linewidth(clw)
         handles.append(Patch(color=c, label=labels[i]))
-    lims = ax.dataLim
-    ax.set_xlim(left=lims.x0-rwidth, right=lims.x1+rwidth)
+    ax.set_xlim(left=-.5, right=len(data)-.5)
     return ax, handles
 
 
 def plot_histo(ax, data, labels, colormap, ylim, stacked=False, rwidth=.8,
-               step=None, bwidth=.9, bins='auto', brange=None, tails=False,
-               ylog=False, density=False, **kwargs):
+               step=None, fcol=0.0, bwidth=.9, bins='auto', brange=None,
+               tails=False, ylog=False, density=False, weights=None, **kwargs):
     """adds histo plots to ax and returns ax and handles for legend"""
     nlab = len(labels)
     for i in range(len(data)):
         if i >= nlab:
             labels.append("series{:02d}".format(i))
-    try:
-        inc = 1./(len(data)-1)
-    except ZeroDivisionError:
-        inc = 1
-    c = [colormap.to_rgba(i*inc) for i in range(len(data))]
+    n = len(data)
+    c = [colormap.to_rgba(color_inc_i(fcol, i, n, step)) for i in range(n)]
     if brange and tails:
         data = [np.clip(np.array(y), brange[0], brange[1]) for y in data]
     plotargs = {'linewidth': 0}
     plotargs.update(kwargs)
-    histo = plt.hist(data, bins=bins, log=ylog, range=brange, label=labels,
+    histo = plt.hist(data, bins=bins, log=ylog, range=brange, label=labels, weights=weights,
                      color=c, rwidth=rwidth, stacked=stacked, density=density, **plotargs)
     handles, labels = ax.get_legend_handles_labels()
     if ylim[1] is None:
@@ -1008,12 +1126,14 @@ def cmap_tune(colormap, step=None, positions=None, name='custom'):
     if positions is not None:
         cmap2 = []
         pos2 = []
-        for i, p in enumerate(positions):
-            inc = i/float(len(positions)-1)
-            cmap2.append(colormap.to_rgba(p))
-            pos2.append(inc)
-        colormap = cmap_from_list(cmap2, None, pos2, name)
-    if step is not None:
+        segments = len(positions) - 1
+        iperseg = int(2000/segments)
+        for i,p in enumerate(positions[0:-1]):
+            inc = (positions[i+1] - p)/iperseg
+            incs = np.arange(p,positions[i+1] + inc, inc)
+            cmap2 += list(colormap.to_rgba(incs))
+        colormap = cmap_from_list(cmap2, step, None, name)
+    elif step is not None:
         cmap2 = []
         for i in range(step):
             inc = i/float(step-1)
